@@ -6,34 +6,35 @@ use base qw(SQL::DB::Schema);
 use Carp qw(carp croak confess);
 use DBI;
 use UNIVERSAL qw(isa);
-use SQL::DB::Schema;
-use SQL::DB::Function;
 use SQL::DB::Row;
 use Class::Accessor::Fast;
 
-use Data::Dumper;
-$Data::Dumper::Indent = 1;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 our $DEBUG   = 0;
-our @EXPORT_OK = @SQL::DB::Function::EXPORT_OK;
 
+our @EXPORT_OK = @SQL::DB::Schema::EXPORT_OK;
 foreach (@EXPORT_OK) {
     no strict 'refs';
-    *{$_} = *{'SQL::DB::Function::'.$_};
+    *{$_} = *{'SQL::DB::Schema::'.$_};
 }
 
+# Define our sequence table
+define_tables([
+    table => 'sqldb',
+    class => 'SQL::DB::Sequence',
+    column => [name => 'name', type => 'VARCHAR(32)', unique => 1],
+    column => [name => 'val', type => 'INTEGER'],
+]);
+
 
 sub new {
     my $proto = shift;
     my $class = ref($proto) || $proto;
     my $self  = bless($class->SUPER::new(@_), $class);
-    $self->define([
-        table => 'sqldb',
-        class => 'SQL::DB::Sequence',
-        column => [name => 'name', type => 'VARCHAR(32)', unique => 1],
-        column => [name => 'val', type => 'INTEGER'],
-    ]);
+    if (!eval {$self->table('sqldb');1;}) {
+        $self->associate_table('sqldb');
+    }
     return $self;
 }
 
@@ -93,8 +94,48 @@ sub dbh {
 sub deploy {
     my $self = shift;
 
-    TABLES: foreach my $table ($self->tables) {
-        my $sth = $self->{sqldb_dbh}->table_info('', '', $table->name, 'TABLE');
+    # calculate which tables reference which other tables, and plan the
+    # deployment order accordingly.
+    my @tables;
+    my %name_to_table = (map {$_->name => $_} $self->tables);
+    my %deployed;
+    my $thash;
+
+    foreach my $t ($self->tables) {
+        my $none = 1;
+        foreach my $c ($t->columns) {
+            if (my $ref = $c->references) {
+                # check for self reference or already deployed
+                next if ($ref->table == $t or $deployed{$ref->name});
+
+                $thash->{$t->name}->{$ref->table->name} = 1;
+                $none = 0;
+            }
+        }
+        
+        if ($none) { # doesn't reference anyone - can deploy first
+            push(@tables, $t);
+            $deployed{$t->name} = 1;
+        }
+    }
+
+    while (my ($t1,$h) =  each %$thash) {
+        while (my ($t2,$val) = each %$h) {
+            next unless($val);
+            if ($deployed{$t2}) { # been deployed, remove this entry
+                delete $h->{$t2};
+            }
+        }
+        if (!keys %$h) {
+            push(@tables, $name_to_table{$t1});
+            $deployed{$t1} = 1;
+            delete $thash->{$t1};
+        }
+    }
+
+
+    TABLES: foreach my $table (@tables) {
+        my $sth = $self->dbh->table_info('', '', $table->name, 'TABLE');
         if (!$sth) {
             die $DBI::errstr;
         }
@@ -109,9 +150,9 @@ sub deploy {
         foreach my $action ($table->sql_create) {
             warn "debug: $action" if($DEBUG);
             my $res;
-            eval {$res = $self->{sqldb_dbh}->do($action);};
+            eval {$res = $self->dbh->do($action);};
             if (!$res or $@) {
-                die $self->{sqldb_dbh}->errstr . ' query: '. $action;
+                die $self->dbh->errstr . ' query: '. $action;
             }
         }
     }
@@ -119,13 +160,111 @@ sub deploy {
 }
 
 
+
+sub query_as_string {
+    my $self = shift;
+    my $sql  = shift || croak 'query_as_string requires an argument';
+    
+    foreach (@_) {
+        if (defined($_) and $_ !~ /^[[:print:]]+$/) {
+            $sql =~ s/\?/*BINARY DATA*/;
+        }
+        else {
+            my $quote = $self->dbh->quote($_);
+            $sql =~ s/\?/$quote/;
+        }
+    }
+    return $sql;
+}
+
+
+sub do {
+    my $self        = shift;
+    my $query       = $self->query(@_);
+    my $rv;
+
+    if ($query->bind_types) { # INSERT or UPDATE
+        eval {
+            my @bind_values = $query->bind_values;
+            my @bind_types  = $query->bind_types;
+            my $count = scalar(@bind_values);
+
+            my $sth = $self->dbh->prepare("$query");
+            foreach my $i (0..($count-1)) {
+                $sth->bind_param($i+1, $bind_values[$i], $bind_types[$i]);
+            }
+            $rv = $sth->execute;
+        };
+    }
+    else { # DELETE
+        eval {
+            $rv = $self->dbh->do("$query", undef, $query->bind_values);
+        };
+    }
+
+    if ($@ or !defined($rv)) {
+        croak "DBI::do $DBI::errstr $@: Query was:\n"
+            . $self->query_as_string("$query", $query->bind_values);
+    }
+
+    carp 'debug: '. $self->query_as_string("$query", $query->bind_values)
+         ." /* Result: $rv */" if($DEBUG);
+
+    $self->{sqldb_qcount}++;
+    return $rv;
+}
+
+
+sub fetch {
+    my $self  = shift;
+    my $query = $self->query(@_);
+    my $class = SQL::DB::Row->make_class_from($query->acolumns);
+
+    if (wantarray) {
+        my $arrayref;
+        eval {
+            $arrayref = $self->dbh->selectall_arrayref(
+                        "$query", undef, $query->bind_values);
+        };
+        if (!$arrayref or $@) {
+            croak "DBI::selectall_arrayref: $DBI::errstr $@: Query was:\n"
+                . $query->_as_string;
+        }
+
+        $self->{sqldb_qcount}++;
+        carp 'debug: (Rows: '. scalar @$arrayref .') '.
+              $self->query_as_string("$query", $query->bind_values) if($DEBUG);
+        return map {$class->new_from_arrayref($_)->_inflate} @{$arrayref};
+    }
+
+    croak 'sorry, cursor support not yet implemented';
+}
+
+
+sub fetch1 {
+    my $self  = shift;
+    my $query = $self->query(@_);
+    my $class = SQL::DB::Row->make_class_from($query->acolumns);
+
+    my @list = $self->dbh->selectrow_array("$query", undef,
+                                                     $query->bind_values);
+    $self->{sqldb_qcount}++;
+    carp 'debug: '. $self->query_as_string("$query", $query->bind_values) if($DEBUG);
+
+    if (@list) {
+        return $class->new_from_arrayref(\@list)->_inflate;
+    }
+    return;
+}
+
+
 sub create_seq {
     my $self = shift;
     my $name = shift || croak 'usage: $db->create_seq($name)';
 
     $self->{sqldb_dbi} || croak 'Must be connected before calling create_seq';
 
-    my $s = SQL::DB::ARow::sqldb->new;
+    my $s = SQL::DB::Schema::ARow::sqldb->new;
 
     if (eval {
         $self->do(
@@ -146,15 +285,15 @@ sub seq {
 
     $self->{sqldb_dbi} || croak 'Must be connected before calling seq';
 
-    $self->{sqldb_dbh}->begin_work;
-    my $sqldb = SQL::DB::ARow::sqldb->new;
+    $self->dbh->begin_work;
+    my $sqldb = SQL::DB::Schema::ARow::sqldb->new;
     my $seq;
     my $no_updates;
 
     eval {
         # Aparent MySQL bug - no locking with FOR UPDATE
         if ($self->{sqldb_dbi} =~ m/mysql/i) {
-            $self->{sqldb_dbh}->do('LOCK TABLES sqldb WRITE, sqldb AS '.
+            $self->dbh->do('LOCK TABLES sqldb WRITE, sqldb AS '.
                                     $sqldb->_alias .' WRITE');
         }
 
@@ -181,234 +320,40 @@ sub seq {
         );
 
         if ($self->{sqldb_dbi} =~ m/mysql/i) {
-            $self->{sqldb_dbh}->do('UNLOCK TABLES');
+            $self->dbh->do('UNLOCK TABLES');
         }
 
     };
 
     if ($@ or !$no_updates) {
         my $tmp = $@;
-        eval {$self->{sqldb_dbh}->rollback;};
+        eval {$self->dbh->rollback;};
 
         if ($self->{sqldb_dbi} =~ m/mysql/i) {
-            $self->{sqldb_dbh}->do('UNLOCK TABLES');
+            $self->dbh->do('UNLOCK TABLES');
         }
 
         croak "seq: $tmp";
     }
 
 
-    $self->{sqldb_dbh}->commit;
+    $self->dbh->commit;
     return $seq->val + 1;
 }
 
 
-# ------------------------------------------------------------------------
-# For everything other than SELECT
-# ------------------------------------------------------------------------
-
-sub do {
-    my $self        = shift;
-    my $query       = $self->query(@_);
-
-    my $rv;
-    eval {
-        $rv = $self->{sqldb_dbh}->do("$query", undef, $query->bind_values);
-    };
-
-    if ($@ or !defined($rv)) {
-        croak "DBI::do $DBI::errstr $@: Query was:\n"
-               . $self->query_as_string("$query", $query->bind_values);
-    }
-
-    $self->{sqldb_qcount}++;
-
-    carp 'debug: '. $self->query_as_string("$query", $query->bind_values)
-         ." /* Result: $rv */" if($DEBUG);
-    return $rv;
-}
-
-
-sub execute {
-    my $self = shift;
-    my ($sql,$attrs,@bind) = @_;
-
-    my $sth;
-    eval {
-        $sth = $self->{sqldb_dbh}->prepare($sql);
-    };
-    if ($@ or !$sth) {
-        croak "DBI::prepare $DBI::errstr $@: Query was:\n"
-              . $self->query_as_string($sql, @bind);
-    }
-
-    my $res;
-    eval {
-        $res = $sth->execute(@bind);
-    };
-    if (!$res or $@) {
-        croak "DBI::execute $DBI::errstr $@: Query was:\n"
-              . $self->query_as_string($sql, @bind);
-    }
-
-    carp 'debug:'. $self->query_as_string($sql, @bind) 
-         ." /* RESULT: $res */" if($DEBUG);
-    $self->{sqldb_qcount}++;
-    return $sth;
-}
-
-
-
-# ------------------------------------------------------------------------
-# SELECT
-# ------------------------------------------------------------------------
-
-
-sub fetch {
-    my $self  = shift;
-    my $query = $self->query(@_);
-    my $class = SQL::DB::Row->make_class_from($query->acolumns);
-
-    if (wantarray) {
-        my $arrayref;
-        eval {
-            $arrayref = $self->{sqldb_dbh}->selectall_arrayref(
-                        "$query", undef, $query->bind_values);
-        };
-        if (!$arrayref or $@) {
-            croak "DBI::selectall_arrayref: $DBI::errstr $@: Query was:\n"
-                . $query->_as_string;
-        }
-
-        $self->{sqldb_qcount}++;
-        carp 'debug: (Rows: '. scalar @$arrayref .') '.
-              $self->query_as_string("$query", $query->bind_values) if($DEBUG);
-        return map {$class->new($_)->_inflate} @{$arrayref};
-    }
-
-    croak 'sorry, cursor support not yet implemented';
-}
-
-
-sub fetch1 {
-    my $self  = shift;
-    my $query = $self->query(@_);
-    my $class = SQL::DB::Row->make_class_from($query->acolumns);
-
-    my @list = $self->{sqldb_dbh}->selectrow_array("$query", undef,
-                                                     $query->bind_values);
-    $self->{sqldb_qcount}++;
-    carp 'debug: '. $self->query_as_string("$query", $query->bind_values) if($DEBUG);
-
-    if (@list) {
-        return $class->new(\@list)->_inflate;
-    }
-    return;
-}
-
-
-sub fetcho {
-    my $self = shift;
-    my $query   = $self->query(@_);
-    my $sth     = $self->execute("$query", undef, $query->bind_values);
-
-    return $self->objects($query, $sth);
-}
-
-
-sub fetcho1 {
-    my $self = shift;
-    my $query   = $self->query(@_);
-    my $sth     = $self->execute($query, undef, $query->bind_values);
-
-    my @results;
-    @results = $self->objects($query, $sth);
-    return $results[0];
-}
-
-
-sub objects {
-    my $self    = shift;
-    my $query   = shift;
-    my $sth     = shift;
-    my @acols   = $query->acolumns;
-
-    my @returns;
-
-    while (my $row = $sth->fetchrow_arrayref) {
-        my %objs;
-        my $i = 0;
-        my $first;
-        my @references;
-
-        foreach my $col (map {$_->_column} @acols) {
-            if (!$col or !ref($col)) {
-                confess "Missing column ". $col;
-            }
-            my $class = $col->table->class;
-            if (!$class) {
-                die "Missing class for table ". $col->table->name;
-            }
-            my $set   = 'set_' . $col->name;
-
-            if (!$objs{$class}) {
-                $objs{$class} = $class->new;
-            }
-            my $obj = $objs{$class};
-            $first ||= $obj;
-
-            $obj->$set($row->[$i++]);
-
-            if ($col->primary and defined($row->[$i-1])) {
-                $obj->_in_storage(1);
-            }
-#    warn "$set ".$row->[$i-1];
-            if (my $ref = $col->references) {
-                # FIXME need to check that refclass was part of the query.
-                # otherwise we are creating objects that were not wanted...
-                my $refclass = $ref->table->class;
-                push(@references, $col);
-            }
-        }
-
-        foreach my $r (@references) {
-            next unless($objs{$r->table->class}->_in_storage);
-            if (my $target = $objs{$r->references->table->class}) {
-                my $set = 'set_' . $r->name;
-                if ($target->_in_storage) {
-                    $objs{$r->table->class}->$set($target);
-                }
-                else {
-                    $objs{$r->table->class}->$set(undef); # FIXME: autoload based on key value
-                }
-            }
-        }
-
-        foreach my $o (values %objs) {
-            $o->{_changed} = {};
-        }
-        push(@returns, $first);
-    }
-    die $self->{sqldb_dbh}->errstr if ($self->{sqldb_dbh}->errstr);
-
-    warn 'debug: # returns: '. scalar(@returns) if($DEBUG);
-
-    return @returns;
-}
-
-
 sub insert {
     my $self = shift;
     foreach my $obj (@_) {
         unless(ref($obj) and $obj->can('q_update')) {
             croak "Not an insertable object: $obj";
         }
-        foreach ($obj->q_insert) {
+        my ($arows, @inserts) = $obj->q_insert; # reference hand-holding
+        foreach (@inserts) {
             if (!$self->do(@$_)) {
                 croak 'INSERT for '. ref($obj) . ' object failed';
             }
         }
-#        $obj->_in_storage(1);
     }
     return 1;
 }
@@ -420,7 +365,8 @@ sub update {
         unless(ref($obj) and $obj->can('q_update')) {
             croak "Not an updatable object: $obj";
         }
-        foreach ($obj->q_update) {
+        my ($arows, @updates) = $obj->q_update; # reference hand-holding
+        foreach (@updates) {
             if ($self->do(@$_) != 1) {
                 croak 'UPDATE for '. ref($obj) . ' object failed';
             }
@@ -436,8 +382,8 @@ sub delete {
         unless(ref($obj) and $obj->can('q_update')) {
             croak "Not a deletable object: $obj";
         }
-        $obj->_in_storage || croak "Can only delete items already in storage";
-        foreach ($obj->q_delete) {
+        my ($arows, @deletes) = $obj->q_delete; # reference hand-holding
+        foreach (@deletes) {
             if ($self->do(@$_) != 1) {
                 croak 'DELETE for '. ref($obj) . ' object failed';
             }
@@ -453,23 +399,13 @@ sub qcount {
 }
 
 
-sub query_as_string {
-    my $self = shift;
-    my $sql  = shift || croak 'query_as_string requires an argument';
-    
-    foreach (@_) {
-        my $quote = $self->{sqldb_dbh}->quote($_);
-        $sql =~ s/\?/$quote/;
-    }
-    return $sql;
-}
-
 
 sub disconnect {
     my $self = shift;
-    if ($self->{sqldb_dbh}) {
+    if ($self->dbh) {
         warn 'debug: Disconnecting from DBI' if($DEBUG);
-        $self->{sqldb_dbh}->disconnect;
+        $self->dbh->disconnect;
+        delete $self->{sqldb_dbh};
     }
     return;
 }
@@ -495,19 +431,16 @@ SQL::DB - Perl interface to SQL Databases
 
 =head1 SYNOPSIS
 
-  use SQL::DB qw(max min coalesce count nextval currval setval);
-  my $db = SQL::DB->new();
+  use SQL::DB qw(define_tables max min coalesce count nextval currval setval);
 
-  $db->define([
+  define_tables([
       table  => 'addresses',
       class  => 'Address',
       column => [name => 'id',   type => 'INTEGER', primary => 1],
       column => [name => 'kind', type => 'INTEGER'],
       column => [name => 'city', type => 'INTEGER'],
-#      seq    => [name => 'name', start => 1, increment => 2],
-  ]);
-
-  $db->define([
+  ],
+  [
       table  => 'persons',
       class  => 'Person',
       column => [name => 'id',      type => 'INTEGER', primary => 1],
@@ -522,11 +455,12 @@ SQL::DB - Perl interface to SQL Databases
       index  => 'name',
   ]);
 
+  my $db = SQL::DB->new();
+
   $db->connect('dbi:SQLite:/tmp/sqldbtest.db', 'user', 'pass', {});
   $db->deploy;
 
-
-  my $persons  = $db->arow('persons');
+  my $persons   = $db->arow('persons');
   my $addresses = $db->arow('addresses');
 
   $db->do(
@@ -552,8 +486,9 @@ SQL::DB - Perl interface to SQL Databases
     where  => $persons->age > 40,
   );
 
-  print 'Head count: '. $ans->count_name .' Max age:'.$ans->max_age."\n";
-  # "Head count: 1 Max age:43"
+  # The following prints "Head count: 1 Max age:43"
+  print 'Head count: '. $ans->count_name .
+          ' Max age: '. $ans->max_age ."\n";
 
 
   my @items = $db->fetch(
@@ -566,11 +501,12 @@ SQL::DB - Perl interface to SQL Databases
     limit     => 10,
   );
 
+  # Give me "Homer(43) lives in Springfield"
   foreach my $item (@items) {
       print $item->name, '(',$item->age,') lives in ', $item->city, "\n";
   }
-  # "Homer(43) lives in Springfield"
-  return @items # this line for automatic test
+
+  return @items # this line for the automatic test
 
 =head1 DESCRIPTION
 
@@ -579,22 +515,18 @@ Perl objects and logic operators. It is NOT an Object
 Relational Mapper like L<Class::DBI> and neither is it an abstraction
 such as L<SQL::Abstract>. It falls somewhere inbetween.
 
-The typical workflow is as follows. After creating an B<SQL::DB>
-object you can:
-
-* define() the desired or existing schema (tables and columns)
+After using define_tables() to specify your schema and creating an
+B<SQL::DB> object, the typical workflow is as follows:
 
 * connect() to the database
 
 * deploy() the schema (CREATE TABLEs etc)
 
-* Create one or more "abstract row" objects using arow().
+* Using one or more "abstract rows" obtained via arow() you can
+do() insert, update or delete queries.
 
-* do() insert, update or delete queries defined using the abstract
-row objects.
-
-* fetch() (select) data with queries defined using the abstract row
-objects.
+* Using one or more "abstract rows" obtained via arow() you can
+fetch() (select) data to work with (and possibly modify).
 
 * Repeat the above three steps as needed. Further queries (with a
 higher level of automation) are possible with the objects returned by
@@ -611,16 +543,22 @@ welcome.
 
 For a more complete introduction see L<SQL::DB::Intro>.
 
+=head1 CLASS SUBROUTINES
+
+=head2 define_tables(@definitions)
+
+Define the structure of tables, their columns, and associated indexes.
+@definition is list of ARRAY references as required by
+L<SQL::DB::Schema::Table>. This class subroutine can be called multiple
+times. Will warn if you redefine a table.
+
 =head1 METHODS
 
-=head2 new
+=head2 new(@names)
 
-Create a new B<SQL::DB> object.
-
-=head2 define(@def)
-
-Define the structure of the tables and indexes in the database. @def
-is a list of ARRAY references as required by L<SQL::DB::Table>.
+Create a new B<SQL::DB> object. The optional @names lists the tables
+that this object is to know about. By default all tables defined by
+define_tables() are known.
 
 =head2 connect($dbi, $user, $pass, $attrs)
 
@@ -641,17 +579,23 @@ Returns the L<DBI> database handle we are connected with.
 
 Runs the CREATE TABLE and CREATE INDEX statements necessary to
 create the schema in the database. Will warn on any tables that
-already exist.
+already exist. Table creation is automatically ordered based on column
+references.
 
 =head2 query(@query)
 
-Return an L<SQL::DB::Query> object as defined by @query. This method
+Return an L<SQL::DB::Schema::Query> object as defined by @query. This method
 is useful when creating nested SELECTs, UNIONs, or you can print the
 returned object if you just want to see what the SQL looks like.
 
+=head2 query_as_string($sql, @bind_values)
+
+An internal function for pretty printing SQL queries by inserting the
+bind values into the SQL itself. Returns a string.
+
 =head2 do(@query)
 
-Constructs a L<SQL::DB::Query> object as defined by @query and runs
+Constructs a L<SQL::DB::Schema::Query> object as defined by @query and runs
 that query against the connected database.  Croaks if an error occurs.
 This is the method to use for any statement that doesn't retrieve
 values (eg INSERT, UPDATE and DELETE). Returns whatever value the
@@ -659,7 +603,7 @@ underlying L<DBI>->do call returns.
 
 =head2 fetch(@query)
 
-Constructs an L<SQL::DB::Query> object as defined by @query and runs
+Constructs an L<SQL::DB::Schema::Query> object as defined by @query and runs
 that query against the connected database.  Croaks if an error occurs.
 This method should be used for SELECT-type statements that retrieve
 rows.
@@ -683,21 +627,58 @@ You should only use this method if you know/expect one result.
 
 Returns the number of successful queries that have been run.
 
-The following methods are part of the _very thin_ object layer that is
-part of B<SQL::DB>. The objects returned by fetch() and fetch1() can
-typically be used here. See L<SQL::DB::Row> for more details.
+=head2 create_seq($name)
 
-=head2 insert($sqlobject)
+This (and the seq() method below) are the only attempt that B<SQL::DB>
+makes at cross-database abstraction. create_seq() creates a sequence called
+$name. The sequence is actually just a row in the 'sqldb' table.
 
-A shortcut for $db->do($sqlobject->q_insert).
+Warns if the sequence already exists, returns true if successful.
+
+=head2 seq($name)
+
+Return the next value for the sequence $name. The uniqueness of the
+returned value is assured by locking the appropriate table (or rows in
+the table) as required.
+
+Note that this is not intended as a replacment for auto-incrementing primary
+keys in MySQL/SQLite, or real sequences in PostgreSQL. It is simply an
+ease-of-use mechanism for applications wishing to use a common sequence
+api across multiple databases.
+
+=head2 disconnect
+
+Disconnect from the database. Effectively DBI->disconnect.
+
+=head1 METHODS ON FETCHED OBJECTS
+
+Although B<SQL::DB> is not an ORM system it does comes with a very
+thin object layer. Objects returned by fetch() and fetch1() can be
+modified using their set_* methods, just like a regular ORM system.
+However, the difference here is that the objects fields may map across
+multiple database tables. 
+
+Since the objects keep track of which columns have changed, and they
+also know which columns belong to which tables and which columns are
+primary keys, they can also automatically generate the appropriate
+commands for UPDATE or DELETE statements in order to make matching
+changes in the database.
+
+Of course, the appropriate statements only work if the primary keys have
+been included as part of the fetch(). See the q_update() and q_delete()
+methods in L<SQL::DB::Row> for more details.
 
 =head2 update($sqlobject)
 
-A shortcut for $db->do($sqlobject->q_update).
+Nearly the same as $db->do($sqlobject->q_update).
 
 =head2 delete($sqlobject)
 
-A shortcut for $db->do($sqlobject->q_delete).
+Nearly the same as $db->do($sqlobject->q_delete).
+
+=head2 insert($sqlobject)
+
+Nearly the same as $db->do($sqlobject->q_insert).
 
 =head1 DEBUGGING
 
@@ -708,8 +689,8 @@ queries and other important actions are 'warn'ed to STDERR
 
 L<SQL::Abstract>, L<DBIx::Class>, L<Class::DBI>, L<Tangram>
 
-You can see B<SQL::DB> in action in the L<MySpam> application
-(disclaimer: I am also the author)
+You can see B<SQL::DB> in action in the L<MySpam> application, also
+by the same author.
 
 =head1 AUTHOR
 
